/******************************************************************************/
/*                                                                            */
/*  Console program that demonstates how to use of the routines in HD.cpp     */
/*                                                                            */
/*  Usage: HDTest <n> <d> <simul> <dist> <alg>                                */
/*                                                                            */
/*    n (int):      number of data points                                     */
/*    d (int):      dimension of the space                                    */
/*    simul (int):  number of simulations                                     */
/*    dist (0..1):  distribution from which the data points are sampled       */
/*                0: multivariate standard normal distribution                */
/*                1: uniform distribution on the grid {-2,-1,0,1,2}^d         */
/*    alg (0..2):   algorithm used to compute the halfspace depth             */
/*                0: Recursive algorithm HD_Rec (k = 1)                       */
/*                1: Combinatorial algorithm HD_Comb (k = d-1)                */
/*                2: Combinatorial algorithm HD_Comb2 (k = d-2)               */
/*                                                                            */
/*  The program simualtes a sample of size <n> from the multivariate          */
/*  distribution <dist> in the Euclidean space of dimension <d>.              */
/*  Then the halfspace depth of the origin w.r.t. the simulated data          */
/*  cloud is computed using the algorithm <alg>. This is repeated             */
/*  <simul> times. The depth values are printed as well as the average        */
/*  time (in seconds) used to compute the depth.                              */
/*                                                                            */
/******************************************************************************/

#include <iostream>
#include <iomanip>
#include <string>
#include <random>
#include <functional>
#include "HD.h"

using namespace std;
using namespace HalfspaceDepth;

void Test(int argc, char **argv) {

  function<double(double*, double**, int, int)> HD_Alg[] 
    = { HD_Rec, HD_Comb, HD_Comb2 };
  const string AlgName[] = { "HD_Rec", "HD_Comb", "HD_Comb2" };

  mt19937 gen(1);
  uniform_int_distribution<int> uniform_int(-2, 2);
  normal_distribution<double> normal;
  function<double()> rnd_dist[] 
    = { bind(normal, gen), bind(uniform_int, gen) };
  const string DistName[] = { "standard normal", "uniform(-2,-1,0,1,2)" };

  clock_t totaltime = 0;
  clock_t starttime;
  double laptime = 0;
  double averagetime;

  int nPunkte = 80;
  int dim = 5;
  int nTest = 1;
  int nDistribution = 0;
  int nAlgorithm = 0;

  double **x, *z;
  double depth;

  for (int i = 1; i < argc; i++) {
    string param = argv[i];
    switch (i) {
    case 1: nPunkte = stoi(param); break;
    case 2: dim = stoi(param); break;
    case 3: nTest = stoi(param); break;
    case 4: nDistribution = stoi(param); break;
    case 5: nAlgorithm = stoi(param); break;
    }
  }
  function<double()> rnd = rnd_dist[nDistribution];
  function<double(double*, double**, int, int)> HD = HD_Alg[nAlgorithm];
  
  cout << "Number of points:      " << nPunkte << endl
       << "Dimension:             " << dim << endl
       << "Number of simulations: " << nTest << endl
       << "Distribution:          " << DistName[nDistribution] << endl
       << "Algorithm:             " << AlgName[nAlgorithm] << endl;
    
  x = new double*[nPunkte];
  z = new double[dim];
  for (int i = 0; i < nPunkte; i++) x[i] = new double[dim];
  for (int j = 0; j < dim; j++) z[j] = 0;

  for (int cnt = 0; cnt < nTest; cnt++) {
    for (int i = 0; i < nPunkte; i++)
      for (int j = 0; j < dim; j++) x[i][j] = rnd();

    starttime = clock();
    depth = HD(z, x, nPunkte, dim);
    laptime = clock() - starttime;
    totaltime += laptime;

    cout << " " << setw(3) << cnt + 1 << ":  " 
      << setw(8) << setprecision(4) << depth << endl;
  }
  cout << endl;

  averagetime = (double)totaltime / nTest;
  cout << "Average time (in s):     " << averagetime / CLOCKS_PER_SEC << endl;
}

int main(int argc, char **argv) {
  Test(argc, argv);
}
